/**
 * Classe oggetto per la gestione e la vendita delle pizze.
 * 
 * @author Damiano Macedonio
 * @version 1.0
 */

public class Pizzeria {
	
	/**
	 * Menu associato alla pizzeria, contiene tutte le pizze offerte. 
	 */
	private Pizza[] menu;
	
	/**
	 * Ordini ricevuti dalla pizzeria.
	 */
	private Ordine[] ordini;
	
	/**
	 * Contiene gli ingredienti disponibili per la pizzeria.
	 */
	private Ingrediente[] dispensa;
	
	/**
	 * Conta gli ordini ricevuti. 
	 */
	private int numeroOrdine = 0;
	
	/**
	 * Crea un nuovo oggetto Pizzeria con parametri impostati dall'utente.
	 * 
	 * @param capienzaDispensa numero massimo di ingredienti disponibili
	 * @param capienzaOrdini   numero massimo di ordini
	 * @param capienzaMenu     numero massimo di pizze offerte
	 */
	public Pizzeria(int capienzaDispensa, int capienzaOrdini, int capienzaMenu) {
		menu = new Pizza[capienzaMenu];
		ordini = new Ordine[capienzaOrdini];
		dispensa = new Ingrediente[capienzaDispensa];
	}
	
	
	/**
	 * Aggiunde un ingrediente a <code>dispensa</code>. 
	 * L'ingrediente non viene aggiunt se e' gia' presente oppure <code>dispensa</code> e' piena.
	 * 
	 * @param nome        identificativo dell'ingrediente
	 * @param descrizione caratteristiche dell'ingrediente
	 * @param prezzo      cifra da aggiungere al prezzo della pizza in cui compare l'ingrediente
	 * @return            <code>true</code> se l'inserimento ha avuto successo, <code>false</code> altrimenti
	 */
	public boolean aggiungiIngrediente(String nome, String descrizione, double prezzo) {		
		if (getIngrediente(nome) == null) {// l'ingrediente non e' presente
			int i = 0;
			while (i < dispensa.length && dispensa[i] != null)
				i++;
			if (i < dispensa.length) {
				dispensa[i] = new Ingrediente(nome, prezzo, descrizione);
				return true;
			}
		}
		return false;
	}
	
	/**
	 * Cerca un ingrediente in <code>dispensa</code>.
	 * 
	 * @param  ingrediente nome dell'ingrediente da cercare
	 * @return l'ingrediente se presente, <code>null</code> altrimenti
	 */
	public Ingrediente getIngrediente(String ingrediente) {
		for(Ingrediente i : dispensa)
			if(i != null && ingrediente.equals(i.getNome()))
				return i;
		return null;
	}
	
	/**
	 * Verifica che tutti gli ingredienti passati siano presenti in <code>dispensa</code>.
	 * 
	 * @param  ingredienti  identificativi degli ingredienti da cercare
	 * @return <code>true</code> se tutti gli ingredienti sono presenti, <code>false</code> altrimenti
	 */
	public boolean ingredientiPresenti(String... ingredienti) {
		for(String i : ingredienti)
			if (getIngrediente(i) == null)
				return false;
		return true;
	}
	

	/**
	 * Aggiunge una nuova pizza a <code>menu</code>.
	 * La pizza viene aggiunta solo se non ancora presente in <code>menu</code> 
	 * e tutti gli ingredienti che la compongono sono presenti in <code>dispensa</code>.
	 * 
	 * @param nome        identificativo della nuova pizza
	 * @param ingredienti ingredienti che compongono la pizza
	 * @return            <code>true</code> se la pizza e' stata inserita, <code>false</code> altrimenti
	 */
	public boolean aggiungiPizza(String nome, String... ingredienti) {
		int k = 0;
		while (k < menu.length && menu[k] != null)
			k ++;
		if (nome == null || k == menu.length || getPizza(nome) != null || !ingredientiPresenti(ingredienti))
			return false;
		
		// c'e' ancora spazio nel menu'
		// la pizza non è ancora in menu 
		// ci sono tutti gli ingredienti
		Pizza p = new Pizza(nome);
		for (String i : ingredienti)
			p.aggiungiIngrediente(getIngrediente(i));
		menu[k] = p;
		return true;
	}
	
	/**
	 * Cerca una pizza in <code>menu</code>.
	 * 
	 * @param nome identificativo della pizza da cercare
	 *             Il valore deve essere diverso da <code>null</code>
	 * @return l'oggetto <code>Pizza</code> corrispondente se presente in <code>menu</code>, <code>null</code> altrimenti
	 */
	public Pizza getPizza(String nome) {
		for (Pizza p : menu)
			if (p != null && nome.equals(p.getNome()))
				return p;
		return null;
	}
	
	/**
	 * Verifica che tutte le pizze passate siano presenti in <code>menu</code>.
	 * 
	 * @param pizze identificativi delle pizze da cercare
	 * @return      <code>true</code> se tutte le pizze sono presenti, <code>false</code> altrimenti
	 */
	public boolean pizzePresenti(String... pizze) {
		for(String p : pizze)
			if (p != null && getPizza(p) == null)
				return false;
		return true;
	}
	
	
	/**
	 * Aggiunge a <code>ordini</code> un nuovo ordine con le pizze individuate dagli identificativi passati.
	 * L'ordine viene aggiunto se non e' ancora stato raggiunto il limite massimo degli ordini accettabili e se
	 * tutte le pizze richieste compaiono in <code>menu</code>.
	 * 
	 * @param  pizze identificativi delle pizze da aggiungere
	 * @return il numero dell'ordine se l'inserimento ha avuto successo, <code>-1</code> altrimenti
	 */
	public int ordinaPizze(String... pizze) {
		boolean pizzeAggiunte = false;
		if (numeroOrdine == ordini.length || !pizzePresenti(pizze))
			return -1;

		// non e' ancora stato raggiunto il numero massimo di ordini 
		// le pizze che compaiono nell'ordine sono tutte presenti in menu
		Ordine o = new Ordine();
		for(String p : pizze)
			if (p != null)
				pizzeAggiunte = o.aggiungiPizza(this.getPizza(p));
		if (pizzeAggiunte) {
			ordini[numeroOrdine] = o;		
			return numeroOrdine++;
		}
		return -1;
	}
	
	/**
	 * Genera lo scontrino relativo al <code>numeroOrdine</code>.
	 * 
	 * @param numeroOrdine indetificativo dell'ordine
	 * @return             lo scontrino relativo se l'ordine e' presente, <code>null</code> altrimenti
	 */
	public String generaScontrino(int numeroOrdine) {
		if(numeroOrdine >= 0 && numeroOrdine < this.numeroOrdine)
			return this.ordini[numeroOrdine].toString();
		return null;
	}

	/**
	 * Genera l'elenco degli ingredienti in <code>dispensa</code>.
	 * 
	 * @return l'elenco degli ingredienti in <code>dispensa</code>
	 */
	
	public String elencoIngredienti() {
		String str = "";
		for(Ingrediente i: dispensa)
			if (i != null)
				str += i.getNome() + "\n";
		return str;
	}
	

	/**
	 * Genera l'elenco delle pizze presenti in <code>menu</code>.
	 * 
	 * @return l'elenco delle pizze in <code>menu</code>
	 */
	public String menuPizze(){
		String str= "";
		for(Pizza p : menu)
			if (p != null)
				str += "\n" + p.toString() + "\n";
		return str;
	}
}
